from django.conf import settings
from django.conf.urls import include
from django.conf.urls.static import static
from django.contrib import admin
from django.urls import re_path
from django.views import defaults as default_views
from django.views.generic import TemplateView

from django_structlog_demo_project.home import api_views, ninja_views, views


def uncaught_exception_view(request):
    raise Exception("Uncaught Exception")


urlpatterns = [
    re_path(r"^$", TemplateView.as_view(template_name="pages/home.html"), name="home"),
    re_path(
        r"^success_task$", views.enqueue_successful_task, name="enqueue_successful_task"
    ),
    re_path(r"^failing_task$", views.enqueue_failing_task, name="enqueue_failing_task"),
    re_path(r"^nesting_task$", views.enqueue_nesting_task, name="enqueue_nesting_task"),
    re_path(r"^unknown_task$", views.enqueue_unknown_task, name="enqueue_unknown_task"),
    re_path(
        r"^rejected_task$", views.enqueue_rejected_task, name="enqueue_rejected_task"
    ),
    re_path(r"^raise_exception", views.raise_exception, name="raise_exception"),
    re_path(
        r"^standard_logger", views.log_with_standard_logger, name="standard_logger"
    ),
    re_path(r"^async_view", views.async_view, name="async_view"),
    re_path(r"^api_view$", api_views.home_api_view, name="api_view"),
    re_path(
        r"^about/", TemplateView.as_view(template_name="pages/about.html"), name="about"
    ),
    re_path(r"^revoke_task", views.revoke_task, name="revoke_task"),
    re_path(
        r"^async_streaming_view",
        views.async_streaming_view,
        name="async_streaming_view",
    ),
    re_path(
        r"^sync_streaming_view", views.sync_streaming_view, name="sync_streaming_view"
    ),
    # Django Admin, use {% url 'admin:index' %}
    re_path(settings.ADMIN_URL, admin.site.urls),
    # User management
    re_path(
        r"^users/",
        include("django_structlog_demo_project.users.urls", namespace="users"),
    ),
    re_path(r"^accounts/", include("allauth.urls")),
    re_path("^ninja/", ninja_views.api.urls),
    # Your stuff: custom urls includes go here
] + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)

if settings.DEBUG:
    # This allows the error pages to be debugged during development, just visit
    # these url in browser to see how these error pages look like.
    urlpatterns += [
        re_path(
            r"^400/",
            default_views.bad_request,
            kwargs={"exception": Exception("Bad Request!")},
        ),
        re_path(
            r"^403/",
            default_views.permission_denied,
            kwargs={"exception": Exception("Permission Denied")},
        ),
        re_path(
            r"^404/",
            default_views.page_not_found,
            kwargs={"exception": Exception("Page not Found")},
        ),
        re_path(r"^500/", default_views.server_error),
        re_path(r"^uncaught_exception/", uncaught_exception_view),
    ]



from allauth.account.adapter import DefaultAccountAdapter
from django.conf import settings
from django.http import HttpRequest


class AccountAdapter(DefaultAccountAdapter):
    def is_open_for_signup(self, request: HttpRequest):
        return getattr(settings, "ACCOUNT_ALLOW_REGISTRATION", True)



import pytest

from .. import api_views

pytestmark = pytest.mark.django_db


class TestApiView:
    def test(self, caplog, request_factory):
        response = api_views.home_api_view(request_factory.get("/"))
        assert response.status_code == 200
        assert len(caplog.records) == 1
        record = caplog.records[0]
        assert record.msg["event"] == "This is a rest-framework structured log"



from django.apps import AppConfig


class UsersAppConfig(AppConfig):
    name = "django_structlog_demo_project.users"
    verbose_name = "Users"
    default_auto_field = "django.db.models.AutoField"

    def ready(self):
        # noinspection PyUnresolvedReferences
        from . import signals  # noqa F401



import os
import sys

sys.path.append(os.path.join(os.path.dirname(__file__), ".."))


# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# http://www.sphinx-doc.org/en/master/config

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, os.path.abspath('.'))

# The master toctree document.
master_doc = "index"

# -- Project information -----------------------------------------------------

project = "django-structlog"
copyright = "2019, Jules Robichaud-Gagnon"
author = "Jules Robichaud-Gagnon"


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = ["sphinx.ext.autodoc", "sphinx.ext.doctest"]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]


def get_version(precision):
    import django_structlog

    return ".".join(str(v) for v in django_structlog.VERSION[:precision])


# Full version
release = get_version(3)

# Minor version
version = get_version(2)

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "sphinx_rtd_theme"

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]



""" ``django-structlog`` is a structured logging integration for ``Django`` project using ``structlog``.
"""

name = "django_structlog"

VERSION = (9, 0, 1)

__version__ = ".".join(str(v) for v in VERSION)



# Generated by Django 3.2.14 on 2022-08-02 17:18

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("users", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="user",
            name="first_name",
            field=models.CharField(
                blank=True, max_length=150, verbose_name="first name"
            ),
        ),
    ]



"""
To understand why this file is here, please read:

http://cookiecutter-django.readthedocs.io/en/latest/faq.html#why-is-there-a-django-contrib-sites-directory-in-cookiecutter-django
"""

from django.conf import settings
from django.db import migrations


def update_site_forward(apps, schema_editor):
    """Set site domain and name."""
    site_model = apps.get_model("sites", "Site")
    site_model.objects.update_or_create(
        id=settings.SITE_ID,
        defaults={"domain": "example.com", "name": "django_structlog_demo_project"},
    )


class Migration(migrations.Migration):
    dependencies = [("sites", "0002_alter_domain_unique")]

    operations = [migrations.RunPython(update_site_forward, migrations.RunPython.noop)]



import structlog
from ninja import NinjaAPI, Router
from ninja.security import SessionAuth

api = NinjaAPI(urls_namespace="ninja")
router = Router()

logger = structlog.get_logger(__name__)


# OptionalSessionAuth is a custom authentication class that allows the user to be anonymous
class OptionalSessionAuth(SessionAuth):
    def authenticate(self, request, key):
        return request.user


@router.get("/ninja", url_name="ninja", auth=OptionalSessionAuth())
def ninja(request):
    logger.info("This is a ninja structured log")
    return {"result": "ok"}


api.add_router("", router)



import logging

from django.conf import settings


# noinspection PyPep8Naming
class AppSettings:
    PREFIX = "DJANGO_STRUCTLOG_"

    @property
    def CELERY_ENABLED(self) -> bool:
        return getattr(settings, self.PREFIX + "CELERY_ENABLED", False)

    @property
    def STATUS_4XX_LOG_LEVEL(self) -> int:
        return getattr(settings, self.PREFIX + "STATUS_4XX_LOG_LEVEL", logging.WARNING)

    @property
    def COMMAND_LOGGING_ENABLED(self) -> bool:
        return getattr(settings, self.PREFIX + "COMMAND_LOGGING_ENABLED", False)

    @property
    def USER_ID_FIELD(self) -> str:
        return getattr(settings, self.PREFIX + "USER_ID_FIELD", "pk")


app_settings = AppSettings()



from django.apps import AppConfig

from .app_settings import app_settings


class DjangoStructLogConfig(AppConfig):
    name = "django_structlog"

    def ready(self) -> None:
        if app_settings.CELERY_ENABLED:
            from .celery.receivers import CeleryReceiver

            self._celery_receiver = CeleryReceiver()
            self._celery_receiver.connect_signals()

        if app_settings.COMMAND_LOGGING_ENABLED:
            from .commands import DjangoCommandReceiver

            self._django_command_receiver = DjangoCommandReceiver()
            self._django_command_receiver.connect_signals()



import pytest
from django.conf import settings
from django.urls import resolve, reverse

pytestmark = pytest.mark.django_db


def test_detail(user: settings.AUTH_USER_MODEL):
    route = f"/users/{user.username}/"
    assert reverse("users:detail", kwargs={"username": user.username}) == route
    assert resolve(route).view_name == "users:detail"


def test_detail_username_with_dot():
    route = "/users/foo.bar/"
    assert reverse("users:detail", kwargs={"username": "foo.bar"}) == route
    assert resolve(route).view_name == "users:detail"


def test_list():
    assert reverse("users:list") == "/users/"
    assert resolve("/users/").view_name == "users:list"


def test_update():
    assert reverse("users:update") == "/users/~update/"
    assert resolve("/users/~update/").view_name == "users:update"


def test_redirect():
    assert reverse("users:redirect") == "/users/~redirect/"
    assert resolve("/users/~redirect/").view_name == "users:redirect"



import pytest
from django.conf import settings
from django.test import RequestFactory

from django_structlog_demo_project.users.tests.factories import UserFactory


@pytest.fixture(autouse=True)
def media_storage(settings, tmpdir):
    settings.MEDIA_ROOT = tmpdir.strpath


@pytest.fixture
def user() -> settings.AUTH_USER_MODEL:
    return UserFactory()


@pytest.fixture
def request_factory() -> RequestFactory:
    return RequestFactory()



from django.contrib.auth import forms, get_user_model
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

User = get_user_model()


class UserChangeForm(forms.UserChangeForm):
    class Meta(forms.UserChangeForm.Meta):
        model = User


class UserCreationForm(forms.UserCreationForm):
    error_message = forms.UserCreationForm.error_messages.update(
        {"duplicate_username": _("This username has already been taken.")}
    )

    class Meta(forms.UserCreationForm.Meta):
        model = User

    def clean_username(self):
        username = self.cleaned_data["username"]

        try:
            User.objects.get(username=username)
        except User.DoesNotExist:
            return username

        raise ValidationError(self.error_messages["duplicate_username"])



import asyncio
import logging
import traceback
import uuid
from typing import Any, AsyncGenerator, Awaitable, Generator, Type, cast
from unittest import mock
from unittest.mock import AsyncMock, Mock, patch

import structlog
from django.contrib.auth.models import AnonymousUser, User
from django.contrib.sites.models import Site
from django.contrib.sites.shortcuts import get_current_site
from django.core.exceptions import PermissionDenied
from django.core.signals import got_request_exception
from django.dispatch import receiver
from django.http import (
    Http404,
    HttpRequest,
    HttpResponse,
    HttpResponseForbidden,
    HttpResponseNotFound,
    HttpResponseServerError,
    StreamingHttpResponse,
)
from django.test import RequestFactory, TestCase, override_settings

from django_structlog.middlewares.request import (
    RequestMiddleware,
    async_streaming_content_wrapper,
    get_request_header,
    sync_streaming_content_wrapper,
)
from django_structlog.signals import (
    bind_extra_request_failed_metadata,
    bind_extra_request_finished_metadata,
    bind_extra_request_metadata,
)


class TestRequestMiddleware(TestCase):
    log_results: Any

    def setUp(self) -> None:
        self.factory = RequestFactory()
        self.logger = structlog.getLogger(__name__)
        self.log_results = None
        Site.objects.update_or_create(
            id=1,
            defaults={"domain": "testserver", "name": "django_structlog_demo_project"},
        )

    def tearDown(self) -> None:
        structlog.contextvars.clear_contextvars()

    def test_process_request_without_user(self) -> None:
        mock_response = Mock()
        mock_response.status_code = 200
        expected_uuid = "00000000-0000-0000-0000-000000000000"

        def get_response(_request: HttpRequest) -> HttpResponse:
            with self.assertLogs(__name__, logging.INFO) as log_results:
                self.logger.info("hello")
            self.log_results = log_results
            return mock_response

        request = self.factory.get("/foo")

        middleware = RequestMiddleware(get_response)
        with patch("uuid.UUID.__str__", return_value=expected_uuid):
            middleware(request)

        self.assertEqual(1, len(self.log_results.records))
        record = self.log_results.records[0]

        self.assertEqual("INFO", record.levelname)
        self.assertIn("request_id", record.msg)
        self.assertNotIn("user_id", record.msg)
        self.assertEqual(expected_uuid, record.msg["request_id"])
        with self.assertLogs(__name__, logging.INFO) as log_results:
            self.logger.info("hello")
        self.assertEqual(1, len(log_results.records))
        record = log_results.records[0]
        self.assertNotIn("request_id", record.msg)

    def test_process_request_with_null_user(self) -> None:
        mock_response = Mock()
        mock_response.status_code = 200
        expected_uuid = "00000000-0000-0000-0000-000000000000"

        def get_response(_request: HttpRequest) -> HttpResponse:
            with self.assertLogs(__name__, logging.INFO) as log_results:
                self.logger.info("hello")
            self.log_results = log_results
            return mock_response

        request = self.factory.get("/foo")
        setattr(request, "user", None)

        middleware = RequestMiddleware(get_response)
        with patch("uuid.UUID.__str__", return_value=expected_uuid):
            middleware(request)

        self.assertEqual(1, len(self.log_results.records))
        record = self.log_results.records[0]

        self.assertEqual("INFO", record.levelname)
        self.assertIn("request_id", record.msg)
        self.assertNotIn("user_id", record.msg)
        self.assertEqual(expected_uuid, record.msg["request_id"])
        with self.assertLogs(__name__, logging.INFO) as log_results:
            self.logger.info("hello")
        self.assertEqual(1, len(log_results.records))
        record = log_results.records[0]
        self.assertNotIn("request_id", record.msg)

    def test_process_request_anonymous(self) -> None:
        mock_response = Mock()
        mock_response.status_code = 200
        expected_uuid = "00000000-0000-0000-0000-000000000000"

        def get_response(_request: HttpRequest) -> HttpResponse:
            with self.assertLogs(__name__, logging.INFO) as log_results:
                self.logger.info("hello")
            self.log_results = log_results
            return mock_response

        request = self.factory.get("/foo")
        request.user = AnonymousUser()

        middleware = RequestMiddleware(get_response)
        with patch("uuid.UUID.__str__", return_value=expected_uuid):
            middleware(request)

        self.assertEqual(1, len(self.log_results.records))
        record = self.log_results.records[0]

        self.assertEqual("INFO", record.levelname)
        self.assertIn("request_id", record.msg)
        self.assertEqual(expected_uuid, record.msg["request_id"])
        self.assertIn("user_id", record.msg)
        self.assertIsNone(record.msg["user_id"])
        with self.assertLogs(__name__, logging.INFO) as log_results:
            self.logger.info("hello")
        self.assertEqual(1, len(log_results.records))
        record = log_results.records[0]
        self.assertNotIn("request_id", record.msg)
        self.assertNotIn("user_id", record.msg)

    def test_process_request_user(self) -> None:
        mock_response = Mock()
        mock_response.status_code = 200
        expected_uuid = "00000000-0000-0000-0000-000000000000"

        def get_response(_request: HttpRequest) -> HttpResponse:
            with self.assertLogs(__name__, logging.INFO) as log_results:
                self.logger.info("hello")
            self.log_results = log_results
            return mock_response

        request = self.factory.get("/foo")

        mock_user: Any = User.objects.create()
        request.user = mock_user

        middleware = RequestMiddleware(get_response)
        with patch("uuid.UUID.__str__", return_value=expected_uuid):
            middleware(request)

        self.assertEqual(1, len(self.log_results.records))
        record = self.log_results.records[0]

        self.assertEqual("INFO", record.levelname)
        self.assertIn("request_id", record.msg)
        self.assertEqual(expected_uuid, record.msg["request_id"])
        self.assertIn("user_id", record.msg)
        self.assertIsInstance(record.msg["user_id"], int)
        self.assertEqual(mock_user.id, record.msg["user_id"])
        with self.assertLogs(__name__, logging.INFO) as log_results:
            self.logger.info("hello")
        self.assertEqual(1, len(log_results.records))
        record = log_results.records[0]
        self.assertNotIn("request_id", record.msg)
        self.assertNotIn("user_id", record.msg)

    def test_process_request_user_uuid(self) -> None:
        mock_response = Mock()
        mock_response.status_code = 200
        expected_uuid = "00000000-0000-0000-0000-000000000000"

        def get_response(_request: HttpRequest) -> HttpResponse:
            with self.assertLogs(__name__, logging.INFO) as log_results:
                self.logger.info("hello")
            self.log_results = log_results
            return mock_response

        request = self.factory.get("/foo")

        mock_user: Any = mock.Mock()
        mock_user.pk = uuid.UUID(expected_uuid)
        request.user = mock_user

        middleware = RequestMiddleware(get_response)
        middleware(request)

        self.assertEqual(1, len(self.log_results.records))
        record = self.log_results.records[0]

        self.assertEqual("INFO", record.levelname)
        self.assertIn("user_id", record.msg)
        self.assertIsInstance(record.msg["user_id"], str)
        self.assertEqual(expected_uuid, record.msg["user_id"])

    def test_process_request_user_without_id(self) -> None:
        mock_response = Mock()
        mock_response.status_code = 200

        def get_response(_request: HttpRequest) -> HttpResponse:
            with self.assertLogs(__name__, logging.INFO) as log_results:
                self.logger.info("hello")
            self.log_results = log_results
            return mock_response

        request = self.factory.get("/foo")

        class SimpleUser:
            pass

        request.user = cast(Any, SimpleUser())
        middleware = RequestMiddleware(get_response)
        middleware(request)

        self.assertEqual(1, len(self.log_results.records))
        record = self.log_results.records[0]

        self.assertEqual("INFO", record.levelname)
        self.assertIn("user_id", record.msg)
        self.assertIsNone(record.msg["user_id"])

    def test_log_user_in_request_finished(self) -> None:
        mock_response = Mock()
        mock_response.status_code = 200
        expected_uuid = "00000000-0000-0000-0000-000000000000"

        mock_user: Any = User.objects.create()

        request = self.factory.get("/foo")

        def get_response(_request: HttpRequest) -> HttpResponse:
            request.user = mock_user
            return mock_response

        middleware = RequestMiddleware(get_response)
        with (
            patch("uuid.UUID.__str__", return_value=expected_uuid),
            self.assertLogs(
                "django_structlog.middlewares.request", logging.INFO
            ) as log_results,
        ):
            middleware(request)

        self.assertEqual(2, len(log_results.records))
        record: Any
        record = log_results.records[0]

        self.assertEqual("INFO", record.levelname)
        self.assertEqual("request_started", record.msg["event"])
        self.assertIn("request_id", record.msg)
        self.assertEqual(expected_uuid, record.msg["request_id"])
        self.assertNotIn("user_id", record.msg)

        record = log_results.records[1]

        self.assertEqual("INFO", record.levelname)
        self.assertEqual("request_finished", record.msg["event"])
        self.assertIn("request_id", record.msg)
        self.assertEqual(expected_uuid, record.msg["request_id"])
        self.assertIn("user_id", record.msg)
        self.assertEqual(mock_user.id, record.msg["user_id"])

    def test_log_user_in_request_finished_with_exception(self) -> None:
        mock_response = Mock()
        mock_response.status_code = 200
        expected_uuid = "00000000-0000-0000-0000-000000000000"

        mock_user: Any = User.objects.create()

        request = self.factory.get("/foo")

        def get_response(_request: HttpRequest) -> HttpResponse:
            request.user = mock_user
            try:
                raise exception
            except Exception:
                got_request_exception.send(object, request=request)
                self.exception_traceback = traceback.format_exc()
            return mock_response

        middleware = RequestMiddleware(get_response)
        exception = Exception()

        middleware.get_response = get_response

        with (
            patch("uuid.UUID.__str__", return_value=expected_uuid),
            self.assertLogs(
                "django_structlog.middlewares.request", logging.INFO
            ) as log_results,
        ):
            middleware(request)

        self.assertEqual(2, len(log_results.records))
        record: Any
        record = log_results.records[0]

        self.assertEqual("INFO", record.levelname)
        self.assertEqual("request_started", record.msg["event"])
        self.assertIn("request_id", record.msg)
        self.assertEqual(expected_uuid, record.msg["request_id"])
        self.assertNotIn("user_id", record.msg)

        record = log_results.records[1]

        self.assertEqual("ERROR", record.levelname)
        self.assertEqual("request_failed", record.msg["event"])
        self.assertIn("request_id", record.msg)
        self.assertEqual(expected_uuid, record.msg["request_id"])
        self.assertIn("exception", record.msg)
        self.assertEqual(self.exception_traceback.strip(), record.msg["exception"])
        self.assertIn("user_id", record.msg)
        self.assertEqual(mock_user.id, record.msg["user_id"])

    def test_signal_bind_extra_request_metadata(self) -> None:
        @receiver(bind_extra_request_metadata)
        def receiver_bind_extra_request_metadata(
            sender: Type[Any],
            signal: Any,
            request: Any = None,
            logger: Any = None,
            log_kwargs: Any = None,
            **kwargs: Any,
        ) -> None:
            current_site = get_current_site(request)
            log_kwargs["request_started_log"] = "foo"
            structlog.contextvars.bind_contextvars(domain=current_site.domain)

        mock_response = Mock()
        mock_response.status_code = 200

        def get_response(_request: HttpRequest) -> HttpResponse:
            with self.assertLogs(__name__, logging.INFO) as log_results:
                self.logger.info("hello")
            self.log_results = log_results
            return mock_response

        request = self.factory.get("/foo")

        mock_user: Any = User.objects.create(email="foo@example.com")
        request.user = mock_user

        middleware = RequestMiddleware(get_response)

        with self.assertLogs(
            "django_structlog.middlewares.request", logging.INFO
        ) as django_structlog_results:
            middleware(request)

        self.assertEqual(1, len(self.log_results.records))
        record = self.log_results.records[0]

        self.assertEqual("INFO", record.levelname)
        self.assertIn("request_id", record.msg)
        self.assertEqual("testserver", record.msg["domain"])
        self.assertIn("user_id", record.msg)
        self.assertEqual(mock_user.id, record.msg["user_id"])

        self.assertEqual(2, len(django_structlog_results.records))
        record = django_structlog_results.records[0]
        self.assertEqual("request_started", record.msg["event"])
        self.assertEqual("foo", record.msg["request_started_log"])

        record = django_structlog_results.records[1]
        self.assertEqual("request_finished", record.msg["event"])
        self.assertNotIn("request_started_log", record.msg)

    def test_signal_bind_extra_request_finished_metadata(self) -> None:
        mock_response = Mock()
        mock_response.status_code = 200

        @receiver(bind_extra_request_finished_metadata)
        def receiver_bind_extra_request_finished_metadata(
            sender: Type[Any],
            signal: Any,
            request: Any = None,
            logger: Any = None,
            response: Any = None,
            log_kwargs: Any = None,
        ) -> None:
            self.assertEqual(response, mock_response)
            current_site = get_current_site(request)
            log_kwargs["request_finished_log"] = "foo"
            structlog.contextvars.bind_contextvars(domain=current_site.domain)

        def get_response(_request: HttpRequest) -> HttpResponse:
            return mock_response

        request = self.factory.get("/foo")

        mock_user: Any = User.objects.create(email="foo@example.com")
        request.user = mock_user

        middleware = RequestMiddleware(get_response)

        with self.assertLogs(
            "django_structlog.middlewares.request", logging.INFO
        ) as log_results:
            middleware(request)

        self.assertEqual(2, len(log_results.records))
        record: Any
        record = log_results.records[0]

        self.assertEqual("INFO", record.levelname)
        self.assertIn("event", record.msg)
        self.assertEqual("request_started", record.msg["event"])
        self.assertIn("request_id", record.msg)
        self.assertNotIn("domain", record.msg)
        self.assertIn("user_id", record.msg)
        self.assertEqual(mock_user.id, record.msg["user_id"])
        record = log_results.records[1]

        self.assertEqual("INFO", record.levelname)
        self.assertIn("event", record.msg)
        self.assertEqual("request_finished", record.msg["event"])
        self.assertIn("request_id", record.msg)
        self.assertIn("domain", record.msg)
        self.assertEqual("testserver", record.msg["domain"])
        self.assertIn("user_id", record.msg)
        self.assertEqual(mock_user.id, record.msg["user_id"])
        self.assertIn("request_finished_log", record.msg)
        self.assertEqual("foo", record.msg["request_finished_log"])

    def test_signal_bind_extra_request_failed_metadata(self) -> None:
        expected_exception = Exception()

        @receiver(bind_extra_request_failed_metadata)
        def receiver_bind_extra_request_failed_metadata(
            sender: Type[Any],
            signal: Any,
            request: Any = None,
            response: Any = None,
            logger: Any = None,
            exception: Any = None,
            log_kwargs: Any = None,
        ) -> None:
            self.assertEqual(exception, expected_exception)
            current_site = get_current_site(request)
            log_kwargs["request_failed_log"] = "foo"
            structlog.contextvars.bind_contextvars(domain=current_site.domain)

        request = self.factory.get("/foo")

        mock_user: Any = User.objects.create(email="foo@example.com")

        request.user = mock_user

        def get_response(_request: HttpRequest) -> HttpResponse:
            try:
                raise expected_exception
            except Exception:
                got_request_exception.send(object, request=request)
            return mock_response

        middleware = RequestMiddleware(get_response)

        mock_response = Mock()

        with self.assertLogs(
            "django_structlog.middlewares.request", logging.INFO
        ) as log_results:
            middleware(request)

        self.assertEqual(2, len(log_results.records))

        record: Any
        record = log_results.records[0]

        self.assertEqual("INFO", record.levelname)
        self.assertIn("event", record.msg)
        self.assertEqual("request_started", record.msg["event"])
        self.assertIn("request_id", record.msg)
        self.assertNotIn("domain", record.msg)
        self.assertIn("user_id", record.msg)
        self.assertEqual(mock_user.id, record.msg["user_id"])
        record = log_results.records[1]

        self.assertEqual("ERROR", record.levelname)
        self.assertIn("event", record.msg)
        self.assertEqual("request_failed", record.msg["event"])
        self.assertIn("request_id", record.msg)
        self.assertIn("domain", record.msg)
        self.assertEqual("testserver", record.msg["domain"])
        self.assertIn("user_id", record.msg)
        self.assertEqual(mock_user.id, record.msg["user_id"])
        self.assertIn("request_failed_log", record.msg)
        self.assertEqual("foo", record.msg["request_failed_log"])

    def test_process_request_error(self) -> None:
        expected_uuid = "00000000-0000-0000-0000-000000000000"

        request = self.factory.get("/foo")
        request.user = AnonymousUser()

        def get_response(_request: HttpRequest) -> Any:
            """Simulate an exception"""
            try:
                raise exception
            except Exception:
                got_request_exception.send(object, request=request)
                self.exception_traceback = traceback.format_exc()

        middleware = RequestMiddleware(get_response)

        exception = Exception("This is an exception")

        with (
            patch("uuid.UUID.__str__", return_value=expected_uuid),
            self.assertLogs(
                logging.getLogger("django_structlog"), logging.INFO
            ) as log_results,
        ):
            middleware(request)

        self.assertEqual(2, len(log_results.records))
        record: Any
        record = log_results.records[0]
        self.assertEqual("INFO", record.levelname)
        self.assertIn("request_id", record.msg)
        self.assertEqual(expected_uuid, record.msg["request_id"])
        self.assertIn("user_id", record.msg)
        self.assertIsNone(record.msg["user_id"])

        record = log_results.records[1]
        self.assertEqual("ERROR", record.levelname)
        self.assertIn("request_id", record.msg)
        self.assertEqual(expected_uuid, record.msg["request_id"])
        self.assertIn("user_id", record.msg)
        self.assertIsNone(record.msg["user_id"])

        self.assertIn("code", record.msg)
        self.assertEqual(record.msg["code"], 500)
        self.assertIn("exception", record.msg)
        self.assertEqual(self.exception_traceback.strip(), record.msg["exception"])
        self.assertIn("request", record.msg)

        with self.assertLogs(__name__, logging.INFO) as log_results:
            self.logger.info("hello")
        self.assertEqual(1, len(log_results.records))
        record = log_results.records[0]
        self.assertNotIn("request_id", record.msg)
        self.assertNotIn("user_id", record.msg)
        self.assertFalse(hasattr(request, "_raised_exception"))

    def test_process_request_403_are_processed_as_regular_requests(self) -> None:
        expected_uuid = "00000000-0000-0000-0000-000000000000"

        request = self.factory.get("/foo")
        request.user = AnonymousUser()

        def get_response(_request: HttpRequest) -> HttpResponse:
            try:
                raise exception
            except Exception:
                got_request_exception.send(object, request=request)
            return HttpResponseForbidden()

        middleware = RequestMiddleware(get_response)

        exception = PermissionDenied()

        with (
            patch("uuid.UUID.__str__", return_value=expected_uuid),
            self.assertLogs(
                logging.getLogger("django_structlog"), logging.INFO
            ) as log_results,
        ):
            middleware(request)

        self.assertEqual(2, len(log_results.records))
        record: Any
        record = log_results.records[0]
        self.assertEqual("INFO", record.levelname)
        self.assertIn("request_id", record.msg)
        self.assertEqual(expected_uuid, record.msg["request_id"])
        self.assertIn("user_id", record.msg)
        self.assertIsNone(record.msg["user_id"])

        record = log_results.records[1]
        self.assertEqual("WARNING", record.levelname)
        self.assertIn("request_id", record.msg)
        self.assertEqual(expected_uuid, record.msg["request_id"])
        self.assertIn("user_id", record.msg)
        self.assertIsNone(record.msg["user_id"])

        self.assertIn("code", record.msg)
        self.assertEqual(record.msg["code"], 403)
        self.assertNotIn("exception", record.msg)
        self.assertIn("request", record.msg)

        with self.assertLogs(__name__, logging.INFO) as log_results:
            self.logger.info("hello")
        self.assertEqual(1, len(log_results.records))
        record = log_results.records[0]
        self.assertNotIn("request_id", record.msg)
        self.assertNotIn("user_id", record.msg)
        self.assertFalse(hasattr(request, "_raised_exception"))

    def test_process_request_404_are_processed_as_regular_requests(self) -> None:
        expected_uuid = "00000000-0000-0000-0000-000000000000"

        request = self.factory.get("/foo")
        request.user = AnonymousUser()

        def get_response(_request: HttpRequest) -> HttpResponse:
            try:
                raise exception
            except Exception:
                got_request_exception.send(object, request=request)
            return HttpResponseNotFound()

        middleware = RequestMiddleware(get_response)

        exception = Http404()

        with (
            patch("uuid.UUID.__str__", return_value=expected_uuid),
            self.assertLogs(
                logging.getLogger("django_structlog"), logging.INFO
            ) as log_results,
        ):
            middleware(request)

        self.assertEqual(2, len(log_results.records))
        record: Any
        record = log_results.records[0]
        self.assertEqual("INFO", record.levelname)
        self.assertIn("request_id", record.msg)
        self.assertEqual(expected_uuid, record.msg["request_id"])
        self.assertIn("user_id", record.msg)
        self.assertIsNone(record.msg["user_id"])

        record = log_results.records[1]
        self.assertEqual("WARNING", record.levelname)
        self.assertIn("request_id", record.msg)
        self.assertEqual(expected_uuid, record.msg["request_id"])
        self.assertIn("user_id", record.msg)
        self.assertIsNone(record.msg["user_id"])

        self.assertIn("code", record.msg)
        self.assertEqual(record.msg["code"], 404)
        self.assertNotIn("exception", record.msg)
        self.assertIn("request", record.msg)

        with self.assertLogs(__name__, logging.INFO) as log_results:
            self.logger.info("hello")
        self.assertEqual(1, len(log_results.records))
        record = log_results.records[0]
        self.assertNotIn("request_id", record.msg)
        self.assertNotIn("user_id", record.msg)
        self.assertFalse(hasattr(request, "_raised_exception"))

    @override_settings(DJANGO_STRUCTLOG_STATUS_4XX_LOG_LEVEL=logging.INFO)
    def test_process_request_4XX_can_be_personalized(self) -> None:
        expected_uuid = "00000000-0000-0000-0000-000000000000"

        request = self.factory.get("/foo")
        request.user = AnonymousUser()

        def get_response(_request: HttpRequest) -> HttpResponse:
            try:
                raise exception
            except Exception:
                got_request_exception.send(object, request=request)
            return HttpResponseNotFound()

        middleware = RequestMiddleware(get_response)

        exception = Http404()

        with (
            patch("uuid.UUID.__str__", return_value=expected_uuid),
            self.assertLogs(
                logging.getLogger("django_structlog"), logging.INFO
            ) as log_results,
        ):
            middleware(request)

        self.assertEqual(2, len(log_results.records))
        record: Any
        record = log_results.records[0]
        self.assertEqual("INFO", record.levelname)
        self.assertIn("request_id", record.msg)
        self.assertEqual(expected_uuid, record.msg["request_id"])
        self.assertIn("user_id", record.msg)
        self.assertIsNone(record.msg["user_id"])

        record = log_results.records[1]
        self.assertEqual("INFO", record.levelname)
        self.assertIn("request_id", record.msg)
        self.assertEqual(expected_uuid, record.msg["request_id"])
        self.assertIn("user_id", record.msg)
        self.assertIsNone(record.msg["user_id"])

        self.assertIn("code", record.msg)
        self.assertEqual(record.msg["code"], 404)
        self.assertNotIn("exception", record.msg)
        self.assertIn("request", record.msg)

        with self.assertLogs(__name__, logging.INFO) as log_results:
            self.logger.info("hello")
        self.assertEqual(1, len(log_results.records))
        record = log_results.records[0]
        self.assertNotIn("request_id", record.msg)
        self.assertNotIn("user_id", record.msg)
        self.assertFalse(hasattr(request, "_raised_exception"))

    def test_process_request_500_are_processed_as_errors(self) -> None:
        expected_uuid = "00000000-0000-0000-0000-000000000000"

        request = self.factory.get("/foo")
        request.user = AnonymousUser()

        def get_response(_request: HttpRequest) -> HttpResponse:
            return HttpResponseServerError()

        middleware = RequestMiddleware(get_response)

        with (
            patch("uuid.UUID.__str__", return_value=expected_uuid),
            self.assertLogs(
                logging.getLogger("django_structlog"), logging.INFO
            ) as log_results,
        ):
            middleware(request)

        self.assertEqual(2, len(log_results.records))
        record: Any
        record = log_results.records[0]
        self.assertEqual("INFO", record.levelname)
        self.assertIn("request_id", record.msg)
        self.assertEqual(expected_uuid, record.msg["request_id"])
        self.assertIn("user_id", record.msg)
        self.assertIsNone(record.msg["user_id"])

        record = log_results.records[1]
        self.assertEqual("ERROR", record.levelname)
        self.assertIn("request_id", record.msg)
        self.assertEqual(expected_uuid, record.msg["request_id"])
        self.assertIn("user_id", record.msg)
        self.assertIsNone(record.msg["user_id"])

        self.assertIn("code", record.msg)
        self.assertEqual(record.msg["code"], 500)
        self.assertNotIn("exception", record.msg)
        self.assertIn("request", record.msg)

        with self.assertLogs(__name__, logging.INFO) as log_results:
            self.logger.info("hello")
        self.assertEqual(1, len(log_results.records))
        record = log_results.records[0]
        self.assertNotIn("request_id", record.msg)
        self.assertNotIn("user_id", record.msg)
        self.assertFalse(hasattr(request, "_raised_exception"))

    def test_should_log_request_id_from_request_x_request_id_header(self) -> None:
        mock_response = Mock()
        mock_response.status_code = 200
        x_request_id = "my-fake-request-id"

        def get_response(_request: HttpRequest) -> HttpResponse:
            with self.assertLogs(__name__, logging.INFO) as log_results:
                self.logger.info("hello")
            self.log_results = log_results
            return mock_response

        request = RequestFactory(HTTP_X_REQUEST_ID=x_request_id).get("/foo")

        middleware = RequestMiddleware(get_response)
        middleware(request)

        self.assertEqual(1, len(self.log_results.records))
        record: Any
        record = self.log_results.records[0]

        self.assertEqual("INFO", record.levelname)
        self.assertIn("request_id", record.msg)
        self.assertNotIn("user_id", record.msg)
        self.assertEqual(x_request_id, record.msg["request_id"])

    def test_should_log_correlation_id_from_request_x_correlation_id_header(
        self,
    ) -> None:
        mock_response = Mock()
        mock_response.status_code = 200
        x_correlation_id = "my-fake-correlation-id"

        def get_response(_request: HttpRequest) -> HttpResponse:
            with self.assertLogs(__name__, logging.INFO) as log_results:
                self.logger.info("hello")
            self.log_results = log_results
            return mock_response

        request = RequestFactory(HTTP_X_CORRELATION_ID=x_correlation_id).get("/foo")

        middleware = RequestMiddleware(get_response)
        middleware(request)

        self.assertEqual(1, len(self.log_results.records))
        record: Any
        record = self.log_results.records[0]

        self.assertEqual("INFO", record.levelname)
        self.assertIn("request_id", record.msg)
        self.assertNotIn("user_id", record.msg)
        self.assertEqual(x_correlation_id, record.msg["correlation_id"])

    def test_sync_streaming_response(self) -> None:
        def streaming_content() -> Generator[Any, None, None]:  # pragma: no cover
            yield

        mock_response = mock.create_autospec(StreamingHttpResponse)
        mock_response.streaming_content = streaming_content()
        mock_response.is_async = False
        mock_response.status_code = 200

        def get_response(_request: HttpRequest) -> Any:
            return mock_response

        request = RequestFactory().get("/foo")

        middleware = RequestMiddleware(get_response)

        mock_wrapper = AsyncMock()
        with patch(
            "django_structlog.middlewares.request.sync_streaming_content_wrapper",
            return_value=mock_wrapper,
        ) as mock_sync_streaming_response_wrapper:
            response: Any = middleware(request)

        mock_sync_streaming_response_wrapper.assert_called_once()
        self.assertEqual(response.streaming_content, mock_wrapper)

    def test_async_streaming_response(self) -> None:
        async def streaming_content() -> AsyncGenerator[Any, None]:  # pragma: no cover
            yield

        mock_response = mock.create_autospec(StreamingHttpResponse)
        mock_response.streaming_content = streaming_content()
        mock_response.is_async = True
        mock_response.status_code = 200

        def get_response(_request: HttpRequest) -> Any:
            return mock_response

        request = RequestFactory().get("/foo")

        middleware = RequestMiddleware(get_response)

        mock_wrapper = AsyncMock()
        with patch(
            "django_structlog.middlewares.request.async_streaming_content_wrapper",
            return_value=mock_wrapper,
        ) as mock_sync_streaming_response_wrapper:
            response: Any = middleware(request)

        mock_sync_streaming_response_wrapper.assert_called_once()
        self.assertEqual(response.streaming_content, mock_wrapper)

    async def test_async_cancel(self) -> None:
        async def async_get_response(request: HttpRequest) -> Any:
            raise asyncio.CancelledError

        middleware = RequestMiddleware(async_get_response)

        mock_request = Mock()
        with (
            patch(
                "django_structlog.middlewares.RequestMiddleware.prepare"
            ) as mock_prepare,
            patch(
                "django_structlog.middlewares.RequestMiddleware.handle_response"
            ) as mock_handle_response,
            self.assertLogs(
                "django_structlog.middlewares.request", logging.WARNING
            ) as log_results,
        ):
            with self.assertRaises(asyncio.CancelledError):
                await cast(Awaitable[HttpResponse], middleware(mock_request))
        mock_prepare.assert_called_once_with(mock_request)
        mock_handle_response.assert_not_called()
        self.assertEqual(1, len(log_results.records))
        record: Any = log_results.records[0]
        self.assertEqual("request_cancelled", record.msg["event"])


class TestRequestMiddlewareRouter(TestCase):
    async def test_async(self) -> None:
        mock_response = Mock()

        async def async_get_response(request: HttpRequest) -> Any:
            return mock_response

        middleware = RequestMiddleware(async_get_response)

        mock_request = Mock()
        with (
            patch(
                "django_structlog.middlewares.RequestMiddleware.prepare"
            ) as mock_prepare,
            patch(
                "django_structlog.middlewares.RequestMiddleware.handle_response"
            ) as mock_handle_response,
        ):
            response = await cast(Awaitable[HttpResponse], middleware(mock_request))
        self.assertEqual(response, mock_response)
        mock_prepare.assert_called_once_with(mock_request)
        mock_handle_response.assert_called_once_with(mock_request, mock_response)

    def test_sync(self) -> None:
        mock_response = Mock()

        def get_response(request: HttpRequest) -> HttpResponse:
            return mock_response

        middleware = RequestMiddleware(get_response)

        mock_request = Mock()
        with (
            patch(
                "django_structlog.middlewares.RequestMiddleware.prepare"
            ) as mock_prepare,
            patch(
                "django_structlog.middlewares.RequestMiddleware.handle_response"
            ) as mock_handle_response,
        ):
            response = middleware(mock_request)
        self.assertEqual(response, mock_response)
        mock_prepare.assert_called_once_with(mock_request)
        mock_handle_response.assert_called_once_with(mock_request, mock_response)


class TestGetRequestHeader(TestCase):
    def test_django_22_or_higher(self) -> None:
        mock_request = mock.MagicMock(spec=["headers"])
        get_request_header(mock_request, "x-foo-bar", "HTTP_X_FOO_BAR")
        mock_request.headers.get.assert_called_once_with("x-foo-bar")

    def test_django_prior_to_22(self) -> None:
        mock_request = mock.MagicMock(spec=["META"])
        get_request_header(mock_request, "x-foo-bar", "HTTP_X_FOO_BAR")
        mock_request.META.get.assert_called_once_with("HTTP_X_FOO_BAR")


class TestSyncStreamingContentWrapper(TestCase):
    def setUp(self) -> None:
        self.logger = structlog.getLogger(__name__)

    def test_success(self) -> None:
        result = Mock()

        def streaming_content() -> Generator[Any, None, None]:
            self.logger.info("streaming_content")
            yield result

        wrapped_streaming_content = sync_streaming_content_wrapper(
            streaming_content(), {"foo": "bar"}
        )
        with (
            self.assertLogs(__name__, logging.INFO) as streaming_content_log_results,
            self.assertLogs(
                "django_structlog.middlewares.request", logging.INFO
            ) as log_results,
        ):
            self.assertEqual(result, next(wrapped_streaming_content))

        self.assertEqual(1, len(log_results.records))
        record: Any
        record = log_results.records[0]
        self.assertEqual("INFO", record.levelname)
        self.assertEqual("streaming_started", record.msg["event"])
        self.assertIn("foo", record.msg)
        self.assertEqual("bar", record.msg["foo"])

        with self.assertLogs(
            "django_structlog.middlewares.request", logging.INFO
        ) as log_results:
            self.assertRaises(StopIteration, next, wrapped_streaming_content)

        self.assertEqual(1, len(streaming_content_log_results.records))
        record = streaming_content_log_results.records[0]
        self.assertEqual("INFO", record.levelname)
        self.assertIn("foo", record.msg)
        self.assertEqual("bar", record.msg["foo"])

        self.assertEqual(1, len(streaming_content_log_results.records))
        record = log_results.records[0]
        self.assertEqual("INFO", record.levelname)
        self.assertEqual("streaming_finished", record.msg["event"])
        self.assertIn("foo", record.msg)
        self.assertEqual("bar", record.msg["foo"])

    def test_failure(self) -> None:
        result = Mock()

        class CustomException(Exception):
            pass

        exception = CustomException()

        def streaming_content() -> Generator[Any, None, None]:
            self.logger.info("streaming_content")
            yield result
            raise exception

        wrapped_streaming_content = sync_streaming_content_wrapper(
            streaming_content(), {"foo": "bar"}
        )
        with (
            self.assertLogs(__name__, logging.INFO) as streaming_content_log_results,
            self.assertLogs(
                "django_structlog.middlewares.request", logging.INFO
            ) as log_results,
        ):
            self.assertEqual(result, next(wrapped_streaming_content))

        self.assertEqual(1, len(log_results.records))
        record: Any
        record = log_results.records[0]
        self.assertEqual("INFO", record.levelname)
        self.assertEqual("streaming_started", record.msg["event"])
        self.assertIn("foo", record.msg)
        self.assertEqual("bar", record.msg["foo"])

        with self.assertLogs(
            "django_structlog.middlewares.request", logging.INFO
        ) as log_results:
            self.assertRaises(CustomException, next, wrapped_streaming_content)

        self.assertEqual(1, len(streaming_content_log_results.records))
        record = streaming_content_log_results.records[0]
        self.assertEqual("INFO", record.levelname)
        self.assertIn("foo", record.msg)
        self.assertEqual("bar", record.msg["foo"])

        self.assertEqual(1, len(streaming_content_log_results.records))
        record = log_results.records[0]
        self.assertEqual("ERROR", record.levelname)
        self.assertEqual("streaming_failed", record.msg["event"])
        self.assertIn("foo", record.msg)
        self.assertEqual("bar", record.msg["foo"])


class TestASyncStreamingContentWrapper(TestCase):
    def setUp(self) -> None:
        self.logger = structlog.getLogger(__name__)

    async def test_success(self) -> None:
        result = Mock()

        async def streaming_content() -> AsyncGenerator[Any, None]:
            self.logger.info("streaming_content")
            yield result

        wrapped_streaming_content = async_streaming_content_wrapper(
            streaming_content(), {"foo": "bar"}
        )
        with (
            self.assertLogs(__name__, logging.INFO) as streaming_content_log_results,
            self.assertLogs(
                "django_structlog.middlewares.request", logging.INFO
            ) as log_results,
        ):
            self.assertEqual(result, await wrapped_streaming_content.__anext__())

        self.assertEqual(1, len(log_results.records))
        record: Any
        record = log_results.records[0]
        self.assertEqual("INFO", record.levelname)
        self.assertEqual("streaming_started", record.msg["event"])
        self.assertIn("foo", record.msg)
        self.assertEqual("bar", record.msg["foo"])

        with self.assertLogs(
            "django_structlog.middlewares.request", logging.INFO
        ) as log_results:
            with self.assertRaises(StopAsyncIteration):
                await wrapped_streaming_content.__anext__()

        self.assertEqual(1, len(streaming_content_log_results.records))
        record = streaming_content_log_results.records[0]
        self.assertEqual("INFO", record.levelname)
        self.assertIn("foo", record.msg)
        self.assertEqual("bar", record.msg["foo"])

        self.assertEqual(1, len(streaming_content_log_results.records))
        record = log_results.records[0]
        self.assertEqual("INFO", record.levelname)
        self.assertEqual("streaming_finished", record.msg["event"])
        self.assertIn("foo", record.msg)
        self.assertEqual("bar", record.msg["foo"])

    async def test_failure(self) -> None:
        result = Mock()

        class CustomException(Exception):
            pass

        exception = CustomException()

        async def streaming_content() -> AsyncGenerator[Any, None]:
            self.logger.info("streaming_content")
            yield result
            raise exception

        wrapped_streaming_content = async_streaming_content_wrapper(
            streaming_content(), {"foo": "bar"}
        )
        with (
            self.assertLogs(__name__, logging.INFO) as streaming_content_log_results,
            self.assertLogs(
                "django_structlog.middlewares.request", logging.INFO
            ) as log_results,
        ):
            self.assertEqual(result, await wrapped_streaming_content.__anext__())

        self.assertEqual(1, len(log_results.records))
        record: Any
        record = log_results.records[0]
        self.assertEqual("INFO", record.levelname)
        self.assertEqual("streaming_started", record.msg["event"])
        self.assertIn("foo", record.msg)
        self.assertEqual("bar", record.msg["foo"])

        with self.assertLogs(
            "django_structlog.middlewares.request", logging.INFO
        ) as log_results:
            with self.assertRaises(CustomException):
                await wrapped_streaming_content.__anext__()

        self.assertEqual(1, len(streaming_content_log_results.records))
        record = streaming_content_log_results.records[0]
        self.assertEqual("INFO", record.levelname)
        self.assertIn("foo", record.msg)
        self.assertEqual("bar", record.msg["foo"])

        self.assertEqual(1, len(streaming_content_log_results.records))
        record = log_results.records[0]
        self.assertEqual("ERROR", record.levelname)
        self.assertEqual("streaming_failed", record.msg["event"])
        self.assertIn("foo", record.msg)
        self.assertEqual("bar", record.msg["foo"])

    async def test_cancel(self) -> None:
        result = Mock()

        exception = asyncio.CancelledError()

        async def streaming_content() -> AsyncGenerator[Any, None]:
            self.logger.info("streaming_content")
            yield result
            raise exception

        wrapped_streaming_content = async_streaming_content_wrapper(
            streaming_content(), {"foo": "bar"}
        )
        with self.assertLogs(__name__, logging.INFO) as streaming_content_log_results:
            self.assertEqual(result, await wrapped_streaming_content.__anext__())
        with self.assertLogs(
            "django_structlog.middlewares.request", logging.INFO
        ) as log_results:
            with self.assertRaises(asyncio.CancelledError):
                await wrapped_streaming_content.__anext__()

        self.assertEqual(1, len(streaming_content_log_results.records))
        record: Any
        record = streaming_content_log_results.records[0]
        self.assertEqual("INFO", record.levelname)
        self.assertIn("foo", record.msg)
        self.assertEqual("bar", record.msg["foo"])

        self.assertEqual(1, len(streaming_content_log_results.records))
        record = log_results.records[0]
        self.assertEqual("WARNING", record.levelname)
        self.assertEqual("streaming_cancelled", record.msg["event"])
        self.assertIn("foo", record.msg)
        self.assertEqual("bar", record.msg["foo"])



from django.contrib import admin
from django.contrib.auth import admin as auth_admin
from django.contrib.auth import get_user_model

from django_structlog_demo_project.users.forms import UserChangeForm, UserCreationForm

User = get_user_model()


@admin.register(User)
class UserAdmin(auth_admin.UserAdmin):
    form = UserChangeForm
    add_form = UserCreationForm
    fieldsets = (("User", {"fields": ("name",)}),) + auth_admin.UserAdmin.fieldsets
    list_display = ["username", "name", "is_superuser"]
    search_fields = ["name"]



from typing import Any

from celery import bootsteps

from .receivers import CeleryReceiver


class DjangoStructLogInitStep(bootsteps.Step):
    """``celery`` worker boot step to initialize ``django_structlog``.

    >>> from celery import Celery
    >>> from django_structlog.celery.steps import DjangoStructLogInitStep
    >>>
    >>> app = Celery("django_structlog_demo_project")
    >>> app.steps['worker'].add(DjangoStructLogInitStep)

    """

    def __init__(self, parent: Any, **kwargs: Any) -> None:
        super().__init__(parent, **kwargs)
        self.receiver = CeleryReceiver()
        self.receiver.connect_worker_signals()



import uuid
from typing import TYPE_CHECKING, Any, List, Mapping, Tuple, Type

import structlog
from django_extensions.management.signals import (  # type: ignore[import-untyped]
    post_command,
    pre_command,
)

if TYPE_CHECKING:  # pragma: no cover
    import contextvars

logger = structlog.getLogger(__name__)


class DjangoCommandReceiver:
    stack: List[Tuple[str, Mapping[str, "contextvars.Token[Any]"]]]

    def __init__(self) -> None:
        self.stack = []

    def pre_receiver(self, sender: Type[Any], *args: Any, **kwargs: Any) -> None:
        command_id = str(uuid.uuid4())
        if len(self.stack):
            parent_command_id, _ = self.stack[-1]
            tokens = structlog.contextvars.bind_contextvars(
                parent_command_id=parent_command_id, command_id=command_id
            )
        else:
            tokens = structlog.contextvars.bind_contextvars(command_id=command_id)
        self.stack.append((command_id, tokens))

        logger.info(
            "command_started",
            command_name=sender.__module__.replace(".management.commands", ""),
        )

    def post_receiver(
        self, sender: Type[Any], outcome: str, *args: Any, **kwargs: Any
    ) -> None:
        logger.info("command_finished")

        if len(self.stack):  # pragma: no branch
            command_id, tokens = self.stack.pop()
            structlog.contextvars.reset_contextvars(**tokens)

    def connect_signals(self) -> None:
        pre_command.connect(self.pre_receiver)
        post_command.connect(self.post_receiver)



import pytest

from .. import celery

pytestmark = pytest.mark.django_db


class TestSuccessfulTask:
    def test(self, caplog):
        celery.successful_task(foo="bar")
        assert len(caplog.records) == 1
        record = caplog.records[0]
        assert record.msg["event"] == "This is a successful task"


class TestFailingTask:
    def test(self):
        with pytest.raises(Exception) as e:
            celery.failing_task(foo="bar")
        assert str(e.value) == "This is a failed task"


class TestNestingTask:
    def test(self, caplog):
        celery.nesting_task()
        assert len(caplog.records) == 1
        record = caplog.records[0]
        assert record.msg["event"] == "This is a nesting task"


class TestNestedTask:
    def test(self, caplog):
        celery.nested_task()
        assert len(caplog.records) == 1
        record = caplog.records[0]
        assert record.msg["event"] == "This is a nested task"


class TestScheduledTask:
    def test(self, caplog):
        celery.scheduled_task()
        assert len(caplog.records) == 1
        record = caplog.records[0]
        assert record.msg["event"] == "This is a scheduled task"


class TestRejectedTask:
    def test(self):
        assert celery.rejected_task() is None


class TestCorruptRejectedTask:
    def test(self, caplog):
        task_id = "11111111-1111-1111-1111-111111111111"
        headers = dict(
            id=task_id,
            task="django_structlog_demo_project.taskapp.celery.rejected_task",
        )
        celery.corrupt_rejected_task(sender=None, headers=headers)
        assert len(caplog.records) == 1
        record = caplog.records[0]
        assert record.msg["event"] == "corrupting rejected_task"
        assert record.msg["task_id"] == task_id
        assert "task" not in headers

    def test_other_tasks_not_corrupted(self, caplog):
        task_id = "11111111-1111-1111-1111-111111111111"
        headers = dict(
            id=task_id,
            task="django_structlog_demo_project.taskapp.celery.successful_task",
        )
        celery.corrupt_rejected_task(sender=None, headers=headers)
        assert len(caplog.records) == 0
        assert (
            headers["task"]
            == "django_structlog_demo_project.taskapp.celery.successful_task"
        )



from typing import TYPE_CHECKING, Any, Optional, Type, cast

import structlog
from celery import current_app
from celery.signals import (
    after_task_publish,
    before_task_publish,
    task_failure,
    task_prerun,
    task_rejected,
    task_retry,
    task_revoked,
    task_success,
    task_unknown,
)

from . import signals

if TYPE_CHECKING:  # pragma: no cover
    from types import TracebackType

logger = structlog.getLogger(__name__)


class CeleryReceiver:
    _priority: Optional[str]

    def __init__(self) -> None:
        self._priority = None

    def receiver_before_task_publish(
        self,
        sender: Optional[Type[Any]] = None,
        headers: Optional[dict[str, Any]] = None,
        body: Optional[dict[str, str]] = None,
        properties: Optional[dict[str, Any]] = None,
        routing_key: Optional[str] = None,
        **kwargs: dict[str, str],
    ) -> None:
        if current_app.conf.task_protocol < 2:
            return

        context = structlog.contextvars.get_merged_contextvars(logger)
        if "task_id" in context:
            context["parent_task_id"] = context.pop("task_id")

        signals.modify_context_before_task_publish.send(
            sender=self.receiver_before_task_publish,
            context=context,
            task_routing_key=routing_key,
            task_properties=properties,
        )
        if properties:
            self._priority = properties.get("priority", None)
        cast(dict[str, Any], headers)["__django_structlog__"] = context

    def receiver_after_task_publish(
        self,
        sender: Optional[Type[Any]] = None,
        headers: Optional[dict[str, Optional[str]]] = None,
        body: Optional[dict[str, Optional[str]]] = None,
        routing_key: Optional[str] = None,
        **kwargs: Any,
    ) -> None:
        properties = {}
        if self._priority is not None:
            properties["priority"] = self._priority
            self._priority = None

        logger.info(
            "task_enqueued",
            child_task_id=(
                headers.get("id")
                if headers
                else cast(dict[str, Optional[str]], body).get("id")
            ),
            child_task_name=(
                headers.get("task")
                if headers
                else cast(dict[str, Optional[str]], body).get("task")
            ),
            routing_key=routing_key,
            **properties,
        )

    def receiver_task_prerun(
        self, task_id: str, task: Any, *args: Any, **kwargs: Any
    ) -> None:
        structlog.contextvars.clear_contextvars()
        structlog.contextvars.bind_contextvars(task_id=task_id)
        metadata = getattr(task.request, "__django_structlog__", {})
        structlog.contextvars.bind_contextvars(**metadata)
        signals.bind_extra_task_metadata.send(
            sender=self.receiver_task_prerun, task=task, logger=logger
        )
        logger.info("task_started", task=task.name)

    def receiver_task_retry(
        self,
        request: Optional[Any] = None,
        reason: Optional[str] = None,
        einfo: Optional[Any] = None,
        **kwargs: Any,
    ) -> None:
        logger.warning("task_retrying", reason=reason)

    def receiver_task_success(
        self, result: Optional[str] = None, **kwargs: Any
    ) -> None:
        signals.pre_task_succeeded.send(
            sender=self.receiver_task_success, logger=logger, result=result
        )
        logger.info("task_succeeded")

    def receiver_task_failure(
        self,
        task_id: Optional[str] = None,
        exception: Optional[Exception] = None,
        traceback: Optional["TracebackType"] = None,
        einfo: Optional[Any] = None,
        sender: Optional[Type[Any]] = None,
        *args: Any,
        **kwargs: Any,
    ) -> None:
        throws = getattr(sender, "throws", ())
        if isinstance(exception, throws):
            logger.info(
                "task_failed",
                error=str(exception),
            )
        else:
            logger.exception(
                "task_failed",
                error=str(exception),
                exception=exception,
            )

    def receiver_task_revoked(
        self,
        request: Any,
        terminated: Optional[bool] = None,
        signum: Optional[Any] = None,
        expired: Optional[Any] = None,
        **kwargs: Any,
    ) -> None:
        metadata = getattr(request, "__django_structlog__", {}).copy()
        metadata["task_id"] = request.id
        metadata["task"] = request.task

        logger.warning(
            "task_revoked",
            terminated=terminated,
            signum=signum.value if signum is not None else None,
            signame=signum.name if signum is not None else None,
            expired=expired,
            **metadata,
        )

    def receiver_task_unknown(
        self,
        message: Optional[str] = None,
        exc: Optional[Exception] = None,
        name: Optional[str] = None,
        id: Optional[str] = None,
        **kwargs: Any,
    ) -> None:
        logger.error(
            "task_not_found",
            task=name,
            task_id=id,
        )

    def receiver_task_rejected(
        self, message: Any, exc: Optional[Exception] = None, **kwargs: Any
    ) -> None:
        logger.exception(
            "task_rejected", task_id=message.properties.get("correlation_id")
        )

    def connect_signals(self) -> None:
        before_task_publish.connect(self.receiver_before_task_publish)
        after_task_publish.connect(self.receiver_after_task_publish)

    def connect_worker_signals(self) -> None:
        before_task_publish.connect(self.receiver_before_task_publish)
        after_task_publish.connect(self.receiver_after_task_publish)
        task_prerun.connect(self.receiver_task_prerun)
        task_retry.connect(self.receiver_task_retry)
        task_success.connect(self.receiver_task_success)
        task_failure.connect(self.receiver_task_failure)
        task_revoked.connect(self.receiver_task_revoked)
        task_unknown.connect(self.receiver_task_unknown)
        task_rejected.connect(self.receiver_task_rejected)



import structlog
from django.core.management import BaseCommand
from django_extensions.management.utils import signalcommand

logger = structlog.getLogger(__name__)


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument("baz", type=str)

    @signalcommand
    def handle(self, baz, *args, **options):
        logger.info("my nested log", baz=baz)
        return 0



from unittest.mock import patch

from django.test import TestCase

from django_structlog.celery import steps


class TestDjangoStructLogInitStep(TestCase):
    def test_call(self) -> None:
        with patch(
            "django_structlog.celery.receivers.CeleryReceiver.connect_worker_signals",
            autospec=True,
        ) as mock_connect:
            step = steps.DjangoStructLogInitStep(None)

        mock_connect.assert_called_once()

        self.assertIsNotNone(step.receiver)



# Generated by Django 3.2.14 on 2022-08-02 17:18

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("sites", "0003_set_site_domain_and_name"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="site",
            options={
                "ordering": ["domain"],
                "verbose_name": "site",
                "verbose_name_plural": "sites",
            },
        ),
    ]



import logging
from typing import Any

import structlog
from django.core.management import BaseCommand, call_command
from django.test import TestCase
from django_extensions.management.utils import (  # type: ignore[import-untyped]
    signalcommand,
)


class TestCommands(TestCase):
    def test_command(self) -> None:
        class Command(BaseCommand):

            @signalcommand  # type: ignore[misc]
            def handle(self, *args: Any, **options: Any) -> Any:
                structlog.getLogger("command").info("command_event")

        with (
            self.assertLogs("command", logging.INFO) as command_log_results,
            self.assertLogs(
                "django_structlog.commands", logging.INFO
            ) as django_structlog_commands_log_results,
        ):
            call_command(Command())

        self.assertEqual(1, len(command_log_results.records))
        record: Any
        record = command_log_results.records[0]
        self.assertEqual("command_event", record.msg["event"])
        self.assertIn("command_id", record.msg)

        self.assertEqual(2, len(django_structlog_commands_log_results.records))
        record = django_structlog_commands_log_results.records[0]
        self.assertEqual("command_started", record.msg["event"])
        self.assertIn("command_id", record.msg)
        record = django_structlog_commands_log_results.records[1]
        self.assertEqual("command_finished", record.msg["event"])
        self.assertIn("command_id", record.msg)

    def test_nested_command(self) -> None:
        class Command(BaseCommand):
            @signalcommand  # type: ignore[misc]
            def handle(self, *args: Any, **options: Any) -> None:
                logger = structlog.getLogger("command")
                logger.info("command_event_1")
                call_command(NestedCommand())
                logger.info("command_event_2")

        class NestedCommand(BaseCommand):
            @signalcommand  # type: ignore[misc]
            def handle(self, *args: Any, **options: Any) -> None:
                structlog.getLogger("nested_command").info("nested_command_event")

        with (
            self.assertLogs("command", logging.INFO) as command_log_results,
            self.assertLogs("nested_command", logging.INFO),
            self.assertLogs(
                "django_structlog.commands", logging.INFO
            ) as django_structlog_commands_log_results,
        ):
            call_command(Command())

        self.assertEqual(2, len(command_log_results.records))
        command_event_1: Any = command_log_results.records[0]
        self.assertEqual("command_event_1", command_event_1.msg["event"])
        self.assertIn("command_id", command_event_1.msg)
        command_event_2: Any = command_log_results.records[1]
        self.assertEqual("command_event_2", command_event_2.msg["event"])
        self.assertIn("command_id", command_event_2.msg)
        self.assertEqual(
            command_event_1.msg["command_id"], command_event_2.msg["command_id"]
        )

        self.assertEqual(4, len(django_structlog_commands_log_results.records))
        command_started_1: Any = django_structlog_commands_log_results.records[0]
        self.assertEqual("command_started", command_started_1.msg["event"])
        self.assertIn("command_id", command_started_1.msg)

        command_started_2: Any = django_structlog_commands_log_results.records[1]
        self.assertEqual("command_started", command_started_2.msg["event"])
        self.assertIn("command_id", command_started_2.msg)
        self.assertIn("parent_command_id", command_started_2.msg)
        self.assertEqual(
            command_started_1.msg["command_id"],
            command_started_2.msg["parent_command_id"],
        )

        command_finished_1: Any = django_structlog_commands_log_results.records[2]
        self.assertEqual("command_finished", command_finished_1.msg["event"])
        self.assertIn("command_id", command_finished_1.msg)
        self.assertIn("parent_command_id", command_finished_1.msg)
        self.assertEqual(
            command_started_1.msg["command_id"],
            command_finished_1.msg["parent_command_id"],
        )

        command_finished_2: Any = django_structlog_commands_log_results.records[3]
        self.assertEqual("command_finished", command_finished_2.msg["event"])
        self.assertIn("command_id", command_finished_2.msg)
        self.assertNotIn("parent_command_id", command_finished_2.msg)
        self.assertEqual(
            command_event_1.msg["command_id"], command_finished_2.msg["command_id"]
        )



from django.test import TestCase

from django_structlog import app_settings


class TestAppSettings(TestCase):
    def test_celery_enabled(self) -> None:
        settings = app_settings.AppSettings()

        with self.settings(DJANGO_STRUCTLOG_CELERY_ENABLED=True):
            self.assertTrue(settings.CELERY_ENABLED)

    def test_celery_disabled(self) -> None:
        settings = app_settings.AppSettings()

        with self.settings(DJANGO_STRUCTLOG_CELERY_ENABLED=False):
            self.assertFalse(settings.CELERY_ENABLED)
